#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/bin/automx-update.de - update or generate new configuration
#
# Copyright (c) 2015-2019  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2015-09-08 jed
# Last Update:  $Id: automx-update.sh 61869 2019-01-01 14:58:52Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib
. /var/install/include/configlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# function: variable is set
#
# input:  $1 - variable name
# return:  0 - variable set
#          1 - variable not set
#------------------------------------------------------------------------------
variable_set ()
{
    local _vs_var1
    local _vs_var2
    local _vs_ret

    eval _vs_var1=\$"${1}"
    eval _vs_var2=\$"{${1}+EMPTY}"

    if [ -z "${_vs_var1}" ]
    then
        if [ "${_vs_var2}" = 'EMPTY' ]
        then
            # variable is empty, but set
            _vs_ret=0
        else
            # variable is not set
            _vs_ret=1
        fi
    else
        # variable has a value
        _vs_ret=0
    fi
    return ${_vs_ret}
}

#------------------------------------------------------------------------------
# print variable only if it has been set
# $1 - variable name
# $2 - comment
#------------------------------------------------------------------------------
printsetvar ()
{
    if variable_set "${1}"
    then
        printvar "${1}" "${2}"
    fi
}

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # fill empty order
    if [ -z "${AUTOMX_ORDER}" ]
    then
        echo "- AUTOMX_ORDER, '' -> 'imap-pop3-smtp'" >> ${_mv_output_file}
        AUTOMX_ORDER="imap-pop3-smtp"
        _mv_modified=1
    fi

    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ -z "${AUTOMX_CERT_NAME}" -a -z "`grep ^AUTOMX_CERT_NAME ${source_conf}`" ]
    then
        echo "- AUTOMX_CERT_NAME=''" >> ${_av_output_file}
        AUTOMX_CERT_NAME=''
        _av_added=1
    fi

    # v0.90.4
    if [ -z "${AUTOMX_CALDAV_ENABLED}" -a -z "`grep ^AUTOMX_CALDAV_ENABLED ${source_conf}`" ]
    then
        echo "- AUTOMX_CALDAV_ENABLED='no'" >> ${_av_output_file}
        AUTOMX_CALDAV_ENABLED='no'
        _av_added=1
    fi

    if [ -z "${AUTOMX_CALDAV_URL}" -a -z "`grep ^AUTOMX_CALDAV_URL ${source_conf}`" ]
    then
        echo "- AUTOMX_CALDAV_URL='https://example.local.lan/'" >> ${_av_output_file}
        AUTOMX_CALDAV_URL='https://example.local.lan/'
        _av_added=1
    fi

    if [ -z "${AUTOMX_CARDDAV_ENABLED}" -a -z "`grep ^AUTOMX_CARDDAV_ENABLED ${source_conf}`" ]
    then
        echo "- AUTOMX_CARDDAV_ENABLED='no'" >> ${_av_output_file}
        AUTOMX_CARDDAV_ENABLED='no'
        _av_added=1
    fi

    if [ -z "${AUTOMX_CARDDAV_URL}" -a -z "`grep ^AUTOMX_CARDDAV_URL ${source_conf}`" ]
    then
        echo "- AUTOMX_CARDDAV_URL='https://example.local.lan/'" >> ${_av_output_file}
        AUTOMX_CARDDAV_URL='https://example.local.lan/'
        _av_added=1
    fi

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for added parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} "deleting old parameters ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl -conf "automx" "2015-09-08" "jed"
        #------------------------------------------------------------------------------

        if [ "${AUTOMX_DO_DEBUG}" = 'yes' -o "${AUTOMX_DO_DEBUG}" = 'no' ]
        then
            # debug active
            printvar "AUTOMX_DO_DEBUG" "debug mode: yes or no"
        else
            # debug parameter exists but not active
            if [ -n "`grep AUTOMX_DO_DEBUG ${source_conf}`" ]
            then
                echo "# AUTOMX_DO_DEBUG='yes'                # debug mode: yes or no"
            fi
        fi

        #------------------------------------------------------------------------------
        printgroup "general settings"
        #------------------------------------------------------------------------------

        printvar    "START_AUTOMX"              "activate configuration: yes or no"
        echo
        printvar    "AUTOMX_ORDER"              "order of mail services"
        printsetvar "AUTOMX_ACCESS_CONTROL_IP"  "allow access from ip address/network"
        printvar    "AUTOMX_CERT_NAME"          "name of service certificate file"
        printsetvar "AUTOMX_DOCUMENT_ROOT"      "custom document root path"
        printsetvar "AUTOMX_APACHE2_PREFIX"     "configuration file name prefix"

        #------------------------------------------------------------------------------
        printgroup "CalDAV/CardDAV settings"
        #------------------------------------------------------------------------------

        if variable_set "AUTOMX_CALDAV_CARDDAV_SERVER"
        then
            printvar "AUTOMX_CALDAV_CARDDAV_SERVER"  "cloud server: nextcloud or owncloud"
            echo
        fi
        printvar    "AUTOMX_CALDAV_ENABLED"        "activate caldav support: yes or no"
        printvar    "AUTOMX_CALDAV_URL"            "server url"
        printvar    "AUTOMX_CARDDAV_ENABLED"       "activate carddav support: yes or no"
        printvar    "AUTOMX_CARDDAV_URL"           "server url"

        #------------------------------------------------------------------------------
        printgroup "log settings"
        #------------------------------------------------------------------------------

        printvar "AUTOMX_LOG_COUNT"             "number of log files to save"
        printvar "AUTOMX_LOG_INTERVAL"          "interval: daily, weekly, monthly"

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------
    } > ${generate_conf}

    # set access rights
    chmod 0644 ${generate_conf}
    chown root ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

configfile=/etc/config.d/automx
installfile=/var/run/automx.install

# setting defaults
source_conf=${installfile}
generate_conf=${configfile}

goflag=0
modflag=0

case "${1}" in
    update )
        goflag=1
        ;;
    test )
      # source_conf=${configfile}.new
        source_conf=${configfile}

        generate_conf=/etc/config.d/automx.test
        goflag=1
        ;;
    * )
        echo
        echo "Use one of the following options:"
        echo
        echo "  automx-update.sh [update] - the file ${configfile}.import will be read, the configuration will"
        echo "                              be checked and an updated configuration file will be written."
        echo
        goflag=0
esac

if [ ${goflag} -eq 1 ]
then
    if [ -f ${source_conf} ]
    then
        # previous configuration file exists

        myecho --pack ${module_name} "previous configuration found ..."
        . ${source_conf}

        rename_variables
        modify_variables
        add_variables
        delete_variables

        create_config
    else
        myecho --pack ${module_name} --error "no configuration ${source_conf} found - exiting."
    fi

    if [ ${modflag} -eq 1 ]
    then
        myanykey --pack ${module_name}
    fi
fi

#==============================================================================
# end
#==============================================================================
exit 0
