#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/automx.sh - configuration apply script for AutoMX
#
# Copyright (c) 2015-2019  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation   : 2015-09-08 jed
# Last Update: $Id: automx.sh 61869 2019-01-01 14:58:52Z jed $
#
# Usage: automx.sh [--check-dns]
#          -check-dns             - check dns names and srv entries
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

EXEC_CMD="$0 $*"

#----------------------------------------------------------------------------------------
# check if apache2 has been enabled
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_apache2 ()
{
    local _cia_ret=1

    # made use of $automxrunfile
    myecho --pack ${module_name} -n "apache2 package: "

    if [ -f ${apache2file} ]
    then
        if [ "${START_APACHE2}" = 'yes' ]
        then
            # apache2 activated
            myecho --pack ${module_name} --info "enabled"
            myecho --pack ${module_name} -n "apache2 SSL support: "

            if [ "${APACHE2_SSL}" = 'yes' ]
            then
                myecho --pack ${module_name} --info "enabled"
                _cia_ret=0
            else
                myecho --pack ${module_name} --info "disabled"
            fi
        else
            # apache2 deactivated
            myecho --pack ${module_name} --warn "disabled"
        fi
    else
        myecho --pack ${module_name} --error "not installed"
    fi

    return ${_cia_ret}
}

#----------------------------------------------------------------------------------------
# get Apache2 module path
# return: module path
#----------------------------------------------------------------------------------------
get_apache2_module_path ()
{
    local _gamp_path='/usr/local/apache2/modules'

    # check which path is valid because it has changed
    # with the introduction of the latest apache2 package.
    for MODPATH in /usr/lib/apache2/modules /usr/local/apache2/modules
    do
        if [ -d ${MODPATH} ]
        then
            if [ -f ${MODPATH}/mod_headers.so ]
            then
                _gamp_path=${MODPATH}
                break
            fi
        fi
    done

    echo "${_gamp_path}"
}

#----------------------------------------------------------------------------------------
# check if apache2-mod_wsgi has been enabled
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_apache2_wsgi ()
{
    local _ciaw_ret=1
    local _ciaw_active
    local _ciaw_start

    # made use of $automxrunfile
    myecho --pack ${module_name} -n "apache2-mod_wsgi package: "

    _ciaw_active=0
    if [ -f ${apache2modwsgifile} ]
    then
        if [ "${START_APACHE2_MOD_WSGI}" = 'no' ]
        then
            # activate package
            sed -i '/^START_APACHE2_MOD_WSGI/s/\(START_APACHE2_MOD_WSGI.*=.\)no\(. .*\)$/\1yes\2/' ${apache2modwsgifile}
            _ciaw_active=1
        fi

        _ciaw_start=`grep "^START_APACHE2_MOD_WSGI=" ${apache2modwsgifile} | sed 's/^START_APACHE2_MOD_WSGI=.\([noyes]*\).*$/\1/'`

        if [ "${_ciaw_start}" = 'yes' ]
        then
            # active
            if [ ${_ciaw_active} -eq 1 ]
            then
                myecho --pack ${module_name} -n --warn "disabled "
                myecho --pack ${module_name} -n "-> "
                myecho --pack ${module_name} --info "enabled"
            else
                myecho --pack ${module_name} --info "disabled"
            fi
            _ciaw_ret=0
        else
            # inactive
            myecho --pack ${module_name} --warn "disabled"
        fi
    else
        myecho --pack ${module_name} --error "not installed"
    fi

    return ${_ciaw_ret}
}

#----------------------------------------------------------------------------------------
# get Apache2 wsgi path
# return: wsgi module path
#----------------------------------------------------------------------------------------
get_apache2_wsgi_path ()
{
    local _gawp_version='2'
    local _gawp_path='/var/lib/apache2-mod_wsgi'

    # check which path is valid because the new apache2-mod_wsgi package
    # supports Python 2 and 3 and this requires a different path to be used.
    _gawp_version=`grep "^APACHE2_MOD_WSGI_PYTHON=" ${apache2modwsgifile} | sed 's/^APACHE2_MOD_WSGI_PYTHON=.\([23]\).*$/\1/'`

    for MODPATH in /var/lib/apache2-mod_wsgi/python${_gawp_version} /var/lib/apache2-mod_wsgi
    do
        if [ -d ${MODPATH} ]
        then
            if [ -f ${MODPATH}/mod_wsgi.so ]
            then
                _gawp_path=${MODPATH}
                break
            fi
        fi
    done

    echo "${_gawp_path}"
}

#----------------------------------------------------------------------------------------
# check if mail has been enabled
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_mail ()
{
    local _cim_ret=1

    # made use of $automxrunfile
    myecho --pack ${module_name} -n "mail package: "

    if [ -f ${mailfile} ]
    then
        if [ "${START_MAIL}" = 'yes' ]
        then
            # mail activated
            myecho --pack ${module_name} --info "enabled"
            _cim_ret=0
        else
            # mail deactivated
            myecho --pack ${module_name} --warn "disabled"
        fi
    else
        myecho --pack ${module_name} --error "not installed"
    fi

    return ${_cim_ret}
}

#----------------------------------------------------------------------------------------
# check if own/nextcloud has been enabled
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_cloud ()
{
    local _cim_ret=1

    # made use of $automxrunfile
    if [ -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
    then
        myecho --pack ${module_name} -n "${AUTOMX_CALDAV_CARDDAV_SERVER} package: "

        if [ "${START_OWNCLOUD}" = 'yes' ]
        then
            # cloud activated
            myecho --pack ${module_name} --info "enabled"
            _cim_ret=0
        else
            # cloud deactivated
            myecho --pack ${module_name} --warn "disabled"
        fi
    else
        myecho --pack ${module_name} -n "cloud package: "
        myecho --pack ${module_name} --warn "none installed"
    fi

    return ${_cim_ret}
}

#------------------------------------------------------------------------------
# check dns resolution
#------------------------------------------------------------------------------
check_dns_resolution ()
{
    local _cdr_ret=0
    local _cdr_domains=''
    local _cdr_domlen
    local _cdr_hostlen
    local _cdr_result
    local _cdr_label
    local _cdr_ip

    if [ \( "${AUTOMX_CALDAV_ENABLED}" = 'yes' -o "${AUTOMX_CARDDAV_ENABLED}" = 'yes' \) -a -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
    then
        _cdr_domains='dav'
    fi

    myecho --pack ${module_name} "checking dns names ..."

    # make sure that table size is aligned to length of hostname
    _cdr_domlen=`expr length  "${SMTP_QUALIFY_DOMAIN}"`
    _cdr_hostlen=`expr ${_cdr_domlen} + 16`

    techo --begin ${_cdr_hostlen} 3 42
    for DNAME in autoconfig autodiscover ${_cdr_domains}
    do
        _cdr_result=`host ${DNAME}.${SMTP_QUALIFY_DOMAIN}`

        if [ $? -eq 0 ]
        then
            _cdr_label='--info'

            echo "${_cdr_result}" | grep -q "alias"

            if [ $? -eq 0 ]
            then
                # alias entry
                _cdr_ip=`echo "${_cdr_result}" | grep "^${DNAME}.${SMTP_QUALIFY_DOMAIN}" | cut -d' ' -f6`
            else
                _cdr_ip=`echo "${_cdr_result}" | cut -d' ' -f4`
            fi
        else
            _cdr_ret=1
            _cdr_ip='not found'
        fi

        if [ "${silent}" != '--quiet' ]
        then
            techo --row "- ${DNAME}.${SMTP_QUALIFY_DOMAIN}" "->" ${_cdr_label} "${_cdr_ip}"
        fi
    done
    techo --end

    return ${_cdr_ret}
}

#------------------------------------------------------------------------------
# check dns srv entries
#------------------------------------------------------------------------------
check_dns_srv_entries ()
{
    local _cdse_ret=0
    local _cdse_srv_entries=''
    local _cdse_domlen
    local _cdse_hostlen
    local _cdse_result
    local _cdse_label
    local _cdse_port
    local _cdse_target

    if [ \( "${AUTOMX_CALDAV_ENABLED}" = 'yes' -o "${AUTOMX_CARDDAV_ENABLED}" = 'yes' \) -a -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
    then
        # _carddavs._tcp 86400 IN SRV 0 1 443 dav.example.com.
        # _carddav._tcp  86400 IN SRV 1 1  80 dav.example.com.
        # _caldavs._tcp 86400 IN SRV 0 1 443 dav.example.com.
        # _caldav._tcp  86400 IN SRV 1 1  80 dav.example.com.
        _cdse_srv_entries='caldavs caldav carddavs carddav'
    fi

    myecho --pack ${module_name} "checking dns srv entries ..."

    # make sure that table size is aligned to length of hostname
    _cdse_domlen=`expr length  "${SMTP_QUALIFY_DOMAIN}"`
    _cdse_hostlen=`expr ${_cdse_domlen} + 22`

    # _submission._tcp     SRV 0 1 587 mail.example.com.
    # _imap._tcp           SRV 2 1 143 imap.example.com.
    # _imaps._tcp          SRV 1 1 993 imap.example.com.
    # _pop3._tcp           SRV 4 1 110 pop3.example.com.
    # _pop3s._tcp          SRV 3 1 995 pop3.example.com.
    techo --begin ${_cdse_hostlen} 3 4r 28
    for SNAME in autodiscover submission imaps imap pop3s pop3 ${_cdse_srv_entries}
    do
        _cdse_result=`host -t SRV _${SNAME}._tcp.${SMTP_QUALIFY_DOMAIN}`

        if [ $? -eq 0 ]
        then
            _cdse_label='--info'
            _cdse_port=`echo "${_cdse_result}" | cut -d' ' -f7`
            _cdse_target=`echo "${_cdse_result}" | cut -d' ' -f8`
        else
            _cdse_ret=1
            _cdse_label='--warn'
            _cdse_port='not'
            _cdse_target='found'
        fi

        if [ "${silent}" != '--quiet' ]
        then
            techo --row "- _${SNAME}._tcp.${SMTP_QUALIFY_DOMAIN}" "->" ${_cdse_label} "${_cdse_port}" ${_cdse_label} "${_cdse_target}"
        fi
    done
    techo --end

    return ${_cdse_ret}
}

#----------------------------------------------------------------------------------
# check if port is accessible
# input : $1 - server name
#         $2 - port number
# return:  0 - successful
#          1 - unsuccessful
#----------------------------------------------------------------------------------
check_port_availabilty ()
{
    local _cpa_sname=$1
    local _cpa_sport=$2

    ${vardir}/bin/check_open_port.pl "${_cpa_sname}" "${_cpa_sport}"

    return $?
}

#------------------------------------------------------------------------------
# check server certificates
#------------------------------------------------------------------------------
check_server_certificate ()
{
    local _csc_ret=0
    local _csc_domains=''

    if [ \( "${AUTOMX_CALDAV_ENABLED}" = 'yes' -o "${AUTOMX_CARDDAV_ENABLED}" = 'yes' \) -a -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
    then
        _csc_domains='dav'
    fi

    for DNAME in autoconfig autodiscover ${_csc_domains}
    do
        myecho --pack ${module_name} -n "checking tls certificate '${DNAME}.${SMTP_QUALIFY_DOMAIN}': "

        if check_port_availabilty "${DNAME}.${SMTP_QUALIFY_DOMAIN}" 443
        then
            echo "GET / HTTP/1.0" | \
                openssl s_client -CApath ${certdir} -servername ${DNAME}.${SMTP_QUALIFY_DOMAIN} \
                                 -connect ${DNAME}.${SMTP_QUALIFY_DOMAIN}:443 ${proxy_connect} 2>/dev/null | \
                openssl x509 -noout -text | grep -q "DNS:${DNAME}.${SMTP_QUALIFY_DOMAIN}"

            if [ $? -eq 0 ]
            then
                myecho --pack ${module_name} --info "verified"
            else
                myecho --pack ${module_name} --warn "invalid"
                _csc_ret=1
            fi
        else
            myecho --pack ${module_name} --error "port 443/tcp unreachable"
            _csc_ret=1
        fi
    done

    return ${_csc_ret}
}

#------------------------------------------------------------------------------
# create run config for restricted users (like wwwrun etc.)
#------------------------------------------------------------------------------
create_run_config ()
{
  # myecho --pack ${module_name} "creating run config ..."

    if [ ! -d ${automxrunpath} ]
    then
        mkdir -p ${automxrunpath}
    fi

    {
        if [ -f ${basefile} ]
        then
            # IP_ETH_N
            # IP_ETH_x_NETWORK
            # IP_ETH_x_NETMASK
            grep -E "^IP_ETH_N|^IP_ETH_[0-9]*_NETWORK|^IP_ETH_[0-9]*_NETMASK" ${basefile} | sed 's/ *#.*$//g'
        fi

        if [ -f ${apache2file} ]
        then
            # apache2 installed
            # START_APACHE2
            # APACHE2_DOCUMENT_ROOT
            # APACHE2_PORT
            # APACHE2_SERVER_ADMIN
            # APACHE2_SERVER_NAME
            # APACHE2_SSL
            # APACHE2_SSL_PORT
            grep -E "^START_APACHE2|^APACHE2_DOCUMENT_ROOT|^APACHE2_PORT|^APACHE2_SERVER_|^APACHE2_SSL" ${apache2file} | sed 's/ *#.*$//g'
        fi

        if [ -f ${apache2modwsgifile} ]
        then
            # apache2-mod_wsgi installed
            grep -E "^START_APACHE2_MOD_WSGI" ${apache2modwsgifile} | sed 's/ *#.*$//g'
        fi

        if [ -f ${mailfile} ]
        then
            # START_MAIL
            # START_POP3
            # START_IMAP
            # POP3IMAP_TRANSPORT
            # MAIL_USER_USE_MAILONLY_PASSWORDS
            # MAIL_USER_N
            # MAIL_USER_x_ACTIVE
            # MAIL_USER_x_USER
            # START_SMTP
            # SMTP_QUALIFY_DOMAIN
            # SMTP_HOSTNAME
            # SMTP_LISTEN_PORT
            # SMTP_LOCAL_DOMAIN_N
            # SMTP_LOCAL_DOMAIN_x
            # SMTP_SERVER_TRANSPORT
            # SMTP_SERVER_SSMTP
            grep -E "^START_|^POP3IMAP_TRANSPORT|^MAIL_USER_USE_MAILONLY_PASSWORDS|^MAIL_USER_N|^MAIL_USER_.*_ACTIVE|^MAIL_USER_.*_USER|^SMTP_QUALIFY_DOMAIN|^SMTP_HOSTNAME|^SMTP_LISTEN_PORT|^SMTP_LOCAL_DOMAIN_|^SMTP_SERVER_TRANSPORT|^SMTP_SERVER_SSMTP" ${mailfile} | sed 's/ *#.*$//g'
        fi

        # START_OWNCLOUD
        case ${AUTOMX_CALDAV_CARDDAV_SERVER} in
            nextcloud )
                grep -E "^START_|^OWNCLOUD_DOCUMENT_ROOT" ${nextcloudfile} | sed 's/ *#.*$//g'
                ;;
            owncloud )
                grep -E "^START_|^OWNCLOUD_DOCUMENT_ROOT" ${owncloudfile} | sed 's/ *#.*$//g'
                ;;
            * )
                ;;
        esac
    } > ${automxrunfile}
}

#----------------------------------------------------------------------------------------
# create apache2 wsgi configuration
#----------------------------------------------------------------------------------------
create_apache2_wsgi_conf ()
{
    local _cawc_mods_file
    local _cawc_network
    local _cawc_netmask
    local _cawc_ret

    myecho --pack ${module_name} "creating apache2 wsgi configuration ..."

    # make sure that no legacy file exists in the directory
    _cawc_mods_file=`dirname ${apache2_mods_enabled_file}`/*${module_name}.conf
    rm -f ${_cawc_mods_file}

    {
        echo '<IfModule !mod_headers.c>'
        echo "  LoadModule headers_module ${apache_mod_headers}"
        echo '</IfModule>'
        echo '<IfModule !mod_rewrite.c>'
        echo "  LoadModule rewrite_module ${apache_mod_rewrite}"
        echo '</IfModule>'
        echo '<IfModule !mod_wsgi.c>'
        echo "  LoadModule wsgi_module ${apache_mod_wsgi}"
        echo '</IfModule>'
        echo
        echo "<VirtualHost autoconfig.${SMTP_QUALIFY_DOMAIN}:${APACHE2_PORT}>"

        # restrict access to service
        if [ -n "${AUTOMX_ACCESS_CONTROL_IP}" ]
        then
            # grant access from custom ip address(es)/ip network(s)
            echo '  <Location />'

            for IPNET in ${AUTOMX_ACCESS_CONTROL_IP}
            do
                echo "${IPNET}" | grep -q "\/"

                if [ $? -eq 0 ]
                then
                    # network address given
                    _cawc_network=`echo "${IPNET}" | cut -d'/' -f1`
                    _cawc_netmask=`echo "${IPNET}" | cut -d'/' -f2`

                    _cawc_ret=0
                    if [ "${_cawc_netmask}" = '0' ]
                    then
                        # world netmask
                        _cawc_netmask='0.0.0.0'
                    else
                        # convert netmask 24 -> 255.255.255.0
                        _cawc_netmask=`netcalc netmask "${_cawc_netmask}"`
                        _cawc_ret=$?
                    fi

                    if [ ${_cawc_ret} -eq 0 ]
                    then
                        echo "    Require ip ${_cawc_network}/${_cawc_netmask}"
                    fi
                else
                    echo "    Require ip ${IPNET}"
                fi
            done

            echo '  </Location>'
        else
            # grant access from default ip address(es)/ip network(s)
            if [ ${IP_ETH_N} -gt 0 ]
            then
                echo '  <Location />'

                for IPNET in ${access_control_ip}
                do
                    echo "${IPNET}" | grep -q "\/"

                    if [ $? -eq 0 ]
                    then
                        # network address given
                        _cawc_network=`echo "${IPNET}" | cut -d'/' -f1`
                        _cawc_netmask=`echo "${IPNET}" | cut -d'/' -f2`

                        _cawc_ret=0
                        if [ "${_cawc_netmask}" = '0' ]
                        then
                            # world netmask
                            _cawc_netmask='0.0.0.0'
                        else
                            # convert netmask 24 -> 255.255.255.0
                            _cawc_netmask=`netcalc netmask "${_cawc_netmask}"`
                            _cawc_ret=$?
                        fi

                        if [ ${_cawc_ret} -eq 0 ]
                        then
                            echo "    Require ip ${_cawc_network}/${_cawc_netmask}"
                        fi
                    else
                        echo "    Require ip ${IPNET}"
                    fi
                done

                echo '  </Location>'
            fi
        fi

        echo "  ServerName   autoconfig.${SMTP_QUALIFY_DOMAIN}:${APACHE2_PORT}"
        echo "  ServerAlias  autodiscover.${SMTP_QUALIFY_DOMAIN}"

        if [ \( "${AUTOMX_CALDAV_ENABLED}" = 'yes' -o "${AUTOMX_CARDDAV_ENABLED}" = 'yes' \) -a -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
        then
            echo "  ServerAlias  dav.${SMTP_QUALIFY_DOMAIN}"
        fi

        echo "  ServerAdmin  ${APACHE2_SERVER_ADMIN}"
        echo "  DocumentRoot ${AUTOMX_DOCUMENT_ROOT}/htdocs"
        echo
        echo '  <IfModule mod_rewrite.c>'
        echo '    RewriteEngine On'
        echo '    RewriteCond %{REQUEST_URI} ^/\.well-known/autoconfig'
      #           RewriteCond %{REQUEST_URI} ^/\.well-known/caldav   [OR]
      #           RewriteCond %{REQUEST_URI} ^/\.well-known/carddav
      #           RewriteRule (.*) / [L,R]
        echo '  </IfModule>'
        echo '  <IfModule mod_wsgi.c>'
        echo '    WSGIChunkedRequest On'
        echo "    WSGIScriptAliasMatch (?i)^/.+/(autodiscover|config-v1.1)\\.xml ${AUTOMX_DOCUMENT_ROOT}/${module_name}/automx_wsgi.py"
        echo "    <Directory \"${AUTOMX_DOCUMENT_ROOT}/htdocs\">"
        echo '      Order allow,deny'
        echo '      Allow from all'
        echo '    </Directory>'
        echo '  </IfModule>'
        echo "  ErrorLog  ${AUTOMX_DOCUMENT_ROOT}/log/error_log"
        echo "  CustomLog ${AUTOMX_DOCUMENT_ROOT}/log/access_log combined"
        echo '</VirtualHost>'

        if [ "${APACHE2_SSL}" = 'yes' ]
        then
            echo
            echo "<VirtualHost autoconfig.${SMTP_QUALIFY_DOMAIN}:${APACHE2_SSL_PORT}>"

            # restrict access to service
            if [ -n "${AUTOMX_ACCESS_CONTROL_IP}" ]
            then
                # grant access from custom ip address(es)/ip network(s)
                echo '  <Location />'

                for IPNET in ${AUTOMX_ACCESS_CONTROL_IP}
                do
                    echo "${IPNET}" | grep -q "\/"

                    if [ $? -eq 0 ]
                    then
                        # network address given
                        _cawc_network=`echo "${IPNET}" | cut -d'/' -f1`
                        _cawc_netmask=`echo "${IPNET}" | cut -d'/' -f2`

                        _cawc_ret=0
                        if [ "${_cawc_netmask}" = '0' ]
                        then
                            # world netmask
                            _cawc_netmask='0.0.0.0'
                        else
                            # convert netmask 24 -> 255.255.255.0
                            _cawc_netmask=`netcalc netmask "${_cawc_netmask}"`
                            _cawc_ret=$?
                        fi

                        if [ ${_cawc_ret} -eq 0 ]
                        then
                            echo "    Require ip ${_cawc_network}/${_cawc_netmask}"
                        fi
                    else
                        echo "    Require ip ${IPNET}"
                    fi
                done

                echo '  </Location>'
            else
                # grant access from default ip address(es)/ip network(s)
                if [ ${IP_ETH_N} -gt 0 ]
                then
                    echo '  <Location />'

                    for IPNET in ${access_control_ip}
                    do
                        echo "${IPNET}" | grep -q "\/"

                        if [ $? -eq 0 ]
                        then
                            # network address given
                            _cawc_network=`echo "${IPNET}" | cut -d'/' -f1`
                            _cawc_netmask=`echo "${IPNET}" | cut -d'/' -f2`

                            _cawc_ret=0
                            if [ "${_cawc_netmask}" = '0' ]
                            then
                                # world netmask
                                _cawc_netmask='0.0.0.0'
                            else
                                # convert netmask 24 -> 255.255.255.0
                                _cawc_netmask=`netcalc netmask "${_cawc_netmask}"`
                                _cawc_ret=$?
                            fi

                            if [ ${_cawc_ret} -eq 0 ]
                            then
                                echo "    Require ip ${_cawc_network}/${_cawc_netmask}"
                            fi
                        else
                            echo "    Require ip ${IPNET}"
                        fi
                    done

                    echo '  </Location>'
                fi
            fi

            echo "  ServerName   autoconfig.${SMTP_QUALIFY_DOMAIN}:${APACHE2_SSL_PORT}"
            echo "  ServerAlias  autodiscover.${SMTP_QUALIFY_DOMAIN}"

            if [ \( "${AUTOMX_CALDAV_ENABLED}" = 'yes' -o "${AUTOMX_CARDDAV_ENABLED}" = 'yes' \) -a -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
            then
                echo "  ServerAlias  dav.${SMTP_QUALIFY_DOMAIN}"
            fi

            echo "  ServerAdmin  ${APACHE2_SERVER_ADMIN}"
            echo "  DocumentRoot ${AUTOMX_DOCUMENT_ROOT}/htdocs"
            echo
            echo '  SSLEngine On'
            echo "  SSLCertificateFile    ${certdir}/${AUTOMX_CERT_NAME}.pem"
            echo "  SSLCertificateKeyFile ${certdir}/${AUTOMX_CERT_NAME}.pem"
            echo "  Header add Strict-Transport-Security \"max-age=15768000; includeSubDomains\""
            echo
            echo '  <IfModule mod_rewrite.c>'
            echo '    RewriteEngine On'
            echo '    RewriteCond %{REQUEST_URI} ^/\.well-known/autoconfig'
          #           RewriteCond %{REQUEST_URI} ^/\.well-known/caldav   [OR]
          #           RewriteCond %{REQUEST_URI} ^/\.well-known/carddav
          #           RewriteRule (.*) / [L,R]
            echo '  </IfModule>'
            echo '  <IfModule mod_wsgi.c>'
            # added /mail/config-v1.1.xml alias to make sure that
            # an answer is provided also on https connections
            echo '    WSGIChunkedRequest On'
            echo "    WSGIScriptAliasMatch (?i)^/.+/(autodiscover|config-v1.1)\\.xml ${AUTOMX_DOCUMENT_ROOT}/${module_name}/automx_wsgi.py"
            echo "    WSGIScriptAlias /mobileconfig ${AUTOMX_DOCUMENT_ROOT}/${module_name}/automx_wsgi.py"
            echo "    <Directory \"${AUTOMX_DOCUMENT_ROOT}/htdocs\">"
            echo '      Order allow,deny'
            echo '      Allow from all'
            echo '    </Directory>'
            echo '  </IfModule>'
            echo "  ErrorLog  ${AUTOMX_DOCUMENT_ROOT}/log/error_log.ssl"
            echo "  CustomLog ${AUTOMX_DOCUMENT_ROOT}/log/access_log.ssl combined"
            echo '</VirtualHost>'
        fi
    } > ${apache2_mods_enabled_file}
}

#------------------------------------------------------------------------------
# imap service configuration
# input: $1 - 'starttls'
#------------------------------------------------------------------------------
create_imap_conf ()
{
    local _cic_config=0
    local _cic_starttls=0
    local _cic_ports

    if [ "$1" = 'starttls' ]
    then
        _cic_starttls=1
        if [ "${POP3IMAP_TRANSPORT}" != 'tls' -a "${POP3IMAP_TRANSPORT}" != 'both' ]
        then
            _cic_config=1
        fi
    fi

    if [ ${_cic_config} -eq 0 ]
    then
        if [ ${_cic_starttls} -eq 1 ]
        then
            # force starttls configuration
            echo 'follow = imap_starttls'
            echo
            echo '[imap_starttls]'
            echo 'backend = static_append'
            echo
            echo '### IMAP-STARTTLS ###'
        else
            echo '### IMAP ###'
        fi

        if [ "${START_IMAP}" = 'yes' ]
        then
            # check if IMAP port is reachable
            if [ ${_cic_starttls} -eq 1 ]
            then
                _cic_ports='143'
            else
                _cic_ports='993 143'
            fi

            for PORT in ${_cic_ports}
            do
                # nc command line switches:
                #  -w  - timeout in seconds
                #  -v  - be verbose
                #  -z  - scan for listening daemons, without sending any data to them
                /usr/bin/nc -w 2 -z ${SMTP_HOSTNAME} ${PORT}

                if [ $? -eq 0 ]
                then
                    imap_port=${PORT}
                    break
                fi
            done

            if [ -n "${imap_port}" ]
            then
                echo 'imap = yes'
                echo "imap_server = ${SMTP_HOSTNAME}"
                echo "imap_port = ${imap_port}"

                # valid options: auto, starttls, ssl, none
                if [ "${POP3IMAP_TRANSPORT}" = 'tls' -o "${POP3IMAP_TRANSPORT}" = 'both' ]
                then
                    if [ "${imap_port}" = '993' ]
                    then
                        # simap
                        echo 'imap_encryption = ssl'
                    else
                        # imaps
                        echo 'imap_encryption = starttls'
                    fi
                else
                    echo 'imap_encryption = none'
                fi

                echo 'imap_auth_identity = ${pop3imap_user}'

                # valid options: plaintext, encrypted, ntlm, gssapi, client-ip-address, tls-client-cert, none
              # echo 'imap_auth = plaintext'
                echo 'imap_auth = encrypted'
              # echo 'imap_expiration_date = 20131231'
                echo 'imap_refresh_ttl = 6'
            else
                echo 'imap = no'
            fi
        else
            echo 'imap = no'
        fi
        echo
    fi
}

#------------------------------------------------------------------------------
# pop3 service configuration
#------------------------------------------------------------------------------
create_pop3_conf ()
{
    local _cpc_pop3_port

    echo '### POP3 ###'

    if [ "${START_POP3}" = 'yes' ]
    then
        # check if POP3 port is reachable
        for PORT in 995 110
        do
            # nc command line switches:
            #  -w  - timeout in seconds
            #  -v  - be verbose
            #  -z  - scan for listening daemons, without sending any data to them
            /usr/bin/nc -w 2 -z ${SMTP_HOSTNAME} ${PORT}

            if [ $? -eq 0 ]
            then
                _cpc_pop3_port=${PORT}
                break
            fi
        done

        if [ -n "${_cpc_pop3_port}" ]
        then
            echo 'pop = yes'
            echo "pop_server = ${SMTP_HOSTNAME}"
            echo "pop_port = ${_cpc_pop3_port}"

            # valid options: auto, starttls, ssl, none
            if [ "${POP3IMAP_TRANSPORT}" = 'tls' -o "${POP3IMAP_TRANSPORT}" = 'both' ]
            then
                if [ "${_cpc_pop3_port}" = '995' ]
                then
                    # spop3
                    echo 'pop_encryption = ssl'
                else
                    # pop3s
                    echo 'pop_encryption = starttls'
                fi
            else
                echo 'pop_encryption = none'
            fi

            echo 'pop_auth_identity = ${pop3imap_user}'

            # valid options: plaintext, encrypted, ntlm, gssapi, client-ip-address, tls-client-cert, none
          # echo 'pop_auth = plaintext'
            echo 'pop_auth = encrypted'
          # echo 'pop_expiration_date = 20131231'
            echo 'pop_refresh_ttl = 6'
        else
            echo 'pop = no'
        fi
    else
        echo 'pop = no'
    fi
    echo
}

#------------------------------------------------------------------------------
# smtp service configuration
#------------------------------------------------------------------------------
create_smtp_conf ()
{
    local _csc_ssmtp_port
    local _csc_smtp_port

    echo '### SMTP ###'

    if [ "${START_SMTP}" = 'yes' ]
    then
        _csc_ssmtp_port=''
        if [ "${SMTP_SERVER_SSMTP}" = 'yes' ]
        then
            _csc_ssmtp_port='465'
        fi

        # check if SMTP port is reachable
        for PORT in 587 ${_csc_ssmtp_port} 25
        do
            # nc command line switches:
            #  -w  - timeout in seconds
            #  -v  - be verbose
            #  -z  - scan for listening daemons, without sending any data to them
            /usr/bin/nc -w 2 -z ${SMTP_HOSTNAME} ${PORT}

            if [ $? -eq 0 ]
            then
                _csc_smtp_port=${PORT}
                break
            fi
        done

        if [ -n "${_csc_smtp_port}" ]
        then
            echo 'smtp = yes'
            echo "smtp_server = ${SMTP_HOSTNAME}"
            echo "smtp_port = ${_csc_smtp_port}"

            # valid options: auto, starttls, ssl, none
            if [ "${SMTP_SERVER_TRANSPORT}" = 'tls' -o "${SMTP_SERVER_TRANSPORT}" = 'both' ]
            then
                if [ "${_csc_smtp_port}" = '465' ]
                then
                    # ssmtp
                    echo 'smtp_encryption = ssl'
                else
                    # smtps
                    echo 'smtp_encryption = starttls'
                fi
            else
                # smtp
                echo 'smtp_encryption = none'
            fi

            echo 'smtp_auth_identity = ${smtp_user}'

            # valid options: plaintext, encrypted, ntlm, gssapi, client-ip-address, tls-client-cert, none, smtp-after-pop
          # echo 'smtp_auth = plaintext'
            echo 'smtp_auth = encrypted'
          # echo 'smtp_expiration_date = 20131231'
            echo 'smtp_refresh_ttl = 6'

            # use this server for all accounts
            echo 'smtp_default = yes'
        else
            echo 'smtp = no'
        fi
    else
        echo 'smtp = no'
    fi
    echo
}

#------------------------------------------------------------------------------
# CalDAV service configuration
#------------------------------------------------------------------------------
create_caldav_conf ()
{
    local _ccc_caldav_proto
    local _ccc_caldav_fqdn
    local _ccc_caldav_port

    echo '### CalDAV ###'

    if [ -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" -a "${AUTOMX_CALDAV_ENABLED}" = 'yes' -a -n "${AUTOMX_CALDAV_URL}" ]
    then
        _ccc_caldav_proto=`echo "${AUTOMX_CALDAV_URL}" | sed 's#^\([^:]*\).*#\1#'`
        _ccc_caldav_fqdn=` echo "${AUTOMX_CALDAV_URL}" | sed 's#[^/]*//\([^@]*@\)\?\([^:/]*\).*#\2#'`
        _ccc_caldav_port=` echo "${AUTOMX_CALDAV_URL}" | sed 's#[^/]*//\([^@]*@\)\?\([^:/]*\):\?\([^/]*\).*#\3#'`

        # check if HTTP  port is reachable
        for PORT in ${_ccc_caldav_port} 443 80
        do
            # nc command line switches:
            #  -w  - timeout in seconds
            #  -v  - be verbose
            #  -z  - scan for listening daemons, without sending any data to them
            /usr/bin/nc -w 2 -z ${_ccc_caldav_fqdn} ${PORT}

            if [ $? -eq 0 ]
            then
                _ccc_caldav_port=${PORT}
                break
            fi
        done

        if [ -n "${_ccc_caldav_port}" ]
        then
            echo 'caldav = yes'
            echo "caldav_server = `echo "${AUTOMX_CALDAV_URL}" | sed 's#[^/]$#/#'`"
            echo 'caldav_auth_identity = ${caldav_user}'
        else
            echo 'caldav = no'
        fi
    else
        echo 'caldav = no'
    fi
    echo
}

#------------------------------------------------------------------------------
# CardDAV service configuration
#------------------------------------------------------------------------------
create_carddav_conf ()
{
    local _ccc_carddav_proto
    local _ccc_carddav_fqdn
    local _ccc_carddav_port

    echo '### CardDAV ###'

    if [ -n "${AUTOMX_CALDAV_CARDDAV_SERVER}" -a "${AUTOMX_CARDDAV_ENABLED}" = 'yes' -a -n "${AUTOMX_CARDDAV_URL}" ]
    then
        _ccc_carddav_proto=`echo "${AUTOMX_CARDDAV_URL}" | sed 's#^\([^:]*\).*#\1#'`
        _ccc_carddav_fqdn=` echo "${AUTOMX_CARDDAV_URL}" | sed 's#[^/]*//\([^@]*@\)\?\([^:/]*\).*#\2#'`
        _ccc_carddav_port=` echo "${AUTOMX_CARDDAV_URL}" | sed 's#[^/]*//\([^@]*@\)\?\([^:/]*\):\?\([^/]*\).*#\3#'`

        # check if HTTP  port is reachable
        for PORT in ${_ccc_carddav_port} 443 80
        do
            # nc command line switches:
            #  -w  - timeout in seconds
            #  -v  - be verbose
            #  -z  - scan for listening daemons, without sending any data to them
            /usr/bin/nc -w 2 -z ${_ccc_carddav_fqdn} ${PORT}

            if [ $? -eq 0 ]
            then
                _ccc_carddav_port=${PORT}
                break
            fi
        done

        if [ -n "${_ccc_carddav_port}" ]
        then
            echo 'carddav = yes'
            echo "carddav_server = `echo "${AUTOMX_CARDDAV_URL}" | sed 's#[^/]$#/#'`"
            echo 'carddav_auth_identity = ${carddav_user}'
        else
            echo 'carddav = no'
        fi
    else
        echo 'carddav = no'
    fi
    echo
}

#------------------------------------------------------------------------------
# create configuration file
#------------------------------------------------------------------------------
create_conf ()
{
    local _cc_dom_list
    local _cc_dom_name
    local _cc_idx
    local _cc_sign_check

    myecho --pack ${module_name} "creating automx configuration file ..."

    {
        echo '#----------------------------------------------------------------------'
        echo "# ${generate_conf} file generated by ${pgmname}"
        echo '#'
        echo "# Creation date: `date`"
        echo '#'
        echo "# Do not edit this file directly, modify '${config_file}' file and"
        echo "# re-run the /var/install/config.d/${pgmname}' command to update."
        echo '#----------------------------------------------------------------------'

        #
        # See also http://manpages.ubuntu.com/manpages/trusty/man5/automx.conf.5.html
        #

        # Controlling automx Runtime Behaviour (mandatory)
        #
        # This section is mandatory - it lists all options controlling automx
        # runtime behaviour. The properties provider and domains are also mandatory.
        #
        # debug        - Debugging is enabled and infos will be sent to logfile.
        # provider     - The FQDN domain name of the domain that provides the
        #                configuration service, e.g. example.com.
        # domains      - Specify a comma separated list of domains automx should
        #                provide autoconfiguration for. The wildcard option * used
        #                in domains instructs automx to answer any configuration
        #                request regardless of the domain sent by the mail client.
        # logfile      - All log information should go to the given file.
        # memcache /   - Statistical data controlling errors caused by clients
        # memcache_ttl   accessing database backends will be sent to the specified
        #                memcache service.
        # client_error_limit - In this example a client may not cause more than 5
        #                      errors before automx will refuse to answer further
        #                      queries.
        # rate_limit_exception_networks - Clients listed in
        #                                 rate_limit_exception_networks are excluded
        #                                 from rate limiting.
        echo
        echo '[automx]'

        if [ "${AUTOMX_DO_DEBUG}" = "yes" ]
        then
            echo 'debug    = yes'
        fi

        echo "logfile  = ${logfile}"
        echo
        echo "provider = ${APACHE2_SERVER_NAME}"

        case ${MAIL_INSTALLED} in
            mail )
                # mail installed
                _cc_dom_list=''
                _cc_idx=1
                while [ ${_cc_idx} -le ${SMTP_LOCAL_DOMAIN_N} ]
                do
                    eval _cc_dom_name='$SMTP_LOCAL_DOMAIN_'${_cc_idx}

                    echo "${_cc_dom_name}" | grep -q '\.'

                    if [ $? -eq 0 ]
                    then
                        if [ -z "${_cc_dom_list}" ]
                        then
                            _cc_dom_list="${_cc_dom_name}"
                        else
                            _cc_dom_list="${_cc_dom_list}, ${_cc_dom_name}"
                        fi
                    fi

                    _cc_idx=`expr ${_cc_idx} + 1`
                done

                if [ -z "`echo "${_cc_dom_list}" | sed 's/ *//g'`" ]
                then
                    _cc_dom_list='example.com'
                fi
                ;;

            * )
                _cc_dom_list='example.com'
                ;;
        esac

        echo "domains  = ${_cc_dom_list}"

        # Protect against DoS
        echo 'memcache     = 127.0.0.1:11211'
        echo 'memcache_ttl = 600'
        echo 'client_error_limit = 20'
        echo 'rate_limit_exception_networks = 127.0.0.0/8, ::1/128'

        # Properties present in all other sections (optional)
        #
        # This section is optional. Settings in this section define properties
        # which will be present in all other sections. It is useful to avoid
        # redundancy. Each section can overwrite settings done here.
        #
        # action - The default action for automx is to provide account settings.
        #          Note: The Microsoft schema forsees other actions that account
        #                provisioning.
        # account_type - The account_type should be an email account.
        # account_name - The account should show up as Example Inc. in the clients
        #                account list.
        # account_name_short - The accounts short name should be Example.
        echo
        echo '[DEFAULT]'
        echo 'action = settings'
        echo 'account_type = email'
        echo 'account_name = eisfair Email Account'
        echo 'account_name_short = eisfair'

        # A global backend (mandatory)
        #
        # Setting this section is mandatory, but it may remain empty. It provides
        # a backend, which will be used whenever automx should serve a domain,
        # but no section with domain-specific settings has been specified.
        #
        # Other sections may either explicitly or implicitly refer to the [global]
        # section as a whole. An explicit reference specifies global as backend
        # property. Implicit references simply announce the domain in automx'
        # domains list and omit an explicit section definition for that domain.
        #
        # Note: This is useful when many domains should use the same backend or
        #       when automx domain property configures it to run as wildcard
        #       service.
        echo
        echo '[global]'
        echo 'backend = script'
        echo 'action  = settings'

        echo 'result_attrs = email_addr, user_desc, local_part, first_name, last_name, pop3imap_user, smtp_user, caldav_user, carddav_user'
        echo "script       = ${check_script} \"%s\""
        # default: whitespace
        echo 'separator    = :'

        # If you want to sign mobileconfig profiles, enable these options.
        # Make sure that your webserver has proper privileges to read the
        # key. The cert file must contain the server certificate and all
        # intermediate certificates. You can simply concatenate these
        # certificates.
      # echo 'sign_mobileconfig = yes'
      # echo 'sign_cert = /path/to/cert'
      # echo 'sign_key = /path/to/key'

        _cc_sign_check=''
        if [ "${CERT_INSTALLED}" = 'certs' ]
        then
            # check if certificate can be used for signing
            # /usr/local/ssl/certs/autoconfig.telejeck.de.pem: OK
            _cc_sign_check=`openssl verify -CApath ${certdir} -purpose smimesign ${certdir}/${AUTOMX_CERT_NAME}.pem 2> /dev/null | \
                            cut -d: -f2 | sed 's/ *//g' | tr '[:upper:]' '[:lower:]'`

            if [ "${_cc_sign_check}" = 'ok' ]
            then
                # example of signing command syntax:
                # openssl smime -sign -nodetach -outform $outform
                #     -aes-256-cbc -signer $sign_cert -inkey $sign_key
                #     -in ./sign-test.txt -out ./sign-test.txt.SIGNED
                echo
                echo 'sign_mobileconfig = yes'
                echo "sign_cert = ${certdir}/${AUTOMX_CERT_NAME}.pem"
                echo "sign_key  = ${certdir}/${AUTOMX_CERT_NAME}.pem"
            else
                _cc_sign_check='fail'
            fi
        fi

        echo
        case ${AUTOMX_ORDER} in
            'imap-pop3-smtp' )
                create_imap_conf
                create_pop3_conf
                create_smtp_conf
                ;;
            'pop3-imap-smtp' )
                create_pop3_conf
                create_imap_conf
                create_smtp_conf
                ;;
            * )
                myecho --pack ${module_name} --force-output --error "- invalid service order, check value of AUTOMX_ORDER parameter."
        esac

        create_caldav_conf
        create_carddav_conf

        if [ "${imap_port}" = '993' ]
        then
            # append imap starttls configuration
            create_imap_conf 'starttls'
        fi
    } > ${generate_config_file}

    if [ "${_cc_sign_check}" = 'fail' ]
    then
        myecho --pack ${module_name} --force-output --warn "- signing of mobile configuration cannot be enabled."
    fi
}

#------------------------------------------------------------------------------
# set specific file access rights
#------------------------------------------------------------------------------
set_access_rights ()
{
    myecho --pack ${module_name} "setting access rights ..."

    # configuration file
    if [ -f ${generate_config_file} ]
    then
        chown root:root ${generate_config_file}
        chmod 0644  ${generate_config_file}
    fi

    # log file
    if [ ! -f ${logfile} ]
    then
        touch ${logfile}
    fi
    chown ${apache_user}:${apache_group} ${logfile}
    chmod 0640 ${logfile}

    # run file
    if [ -f ${automxrunfile} ]
    then
        chown ${apache_user}:${apache_group} ${automxrunfile}
        chmod 0640 ${automxrunfile}
    fi

    # wsgi scripts
    if [ -d ${automxwsgipath} ]
    then
        chown -R root:${apache_group} ${automxwsgipath}

        chmod 0750 ${automxwsgipath}
        find ${automxwsgipath} -type f -exec chmod 0440 '{}' \;
    fi

    # document root
    if [ -d ${AUTOMX_DOCUMENT_ROOT} ]
    then
        chown root:root    ${AUTOMX_DOCUMENT_ROOT}
        chown -R root:root ${AUTOMX_DOCUMENT_ROOT}/log
        chown -R root:${apache_group} ${AUTOMX_DOCUMENT_ROOT}/htdocs

        chmod 0750 ${AUTOMX_DOCUMENT_ROOT}/log
        find ${AUTOMX_DOCUMENT_ROOT}/log -type f -exec chmod 0640 '{}' \;

        find ${AUTOMX_DOCUMENT_ROOT}/htdocs -type d -exec chmod 0750 '{}' \;
        find ${AUTOMX_DOCUMENT_ROOT}/htdocs -type f -exec chmod 0440 '{}' \;
    fi
}

#----------------------------------------------------------------------------------------
# restart apache2 service
#----------------------------------------------------------------------------------------
restart_apache2_service ()
{
    # reload apache2 configuration if apache is running
    if [ -f /var/run/httpd.pid -a -f /etc/init.d/apache2 ]
    then
        /etc/init.d/apache2 stop
        sleep 3
        /etc/init.d/apache2 start
    fi
}

#----------------------------------------------------------------------------------------
# create logrotate file
#----------------------------------------------------------------------------------------
create_logrotate ()
{

    # create directory if doesn't exist
    if [ ! -f ${logdir} ]
    then
        mkdir -p ${logdir}
    fi

    if [ "${START_AUTOMX}" = 'yes' ]
    then
        myecho --pack ${module_name} "creating logrotate configuration ..."

        {
            #----------------------------------------------------------------------------------------
            print_short_header ${logrotatefile} ${pgmname} ${module_name} ${package_version}
            #----------------------------------------------------------------------------------------

            echo "${logfile} {"
            echo "    rotate ${AUTOMX_LOG_COUNT}"
            echo "    ${AUTOMX_LOG_INTERVAL}"
            echo '    compress'
            echo '    delaycompress'
            echo "    create 644 ${apache_user} ${apache_group}"
            echo '    notifempty'
            echo '    }'
            echo
        } > ${logrotatefile}
    else
        myecho --pack ${module_name} "disabling logrotate configuration ..."
        rm -f ${logrotatefile}
    fi
}

#========================================================================================
# main
#========================================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

# file names and pathes
automxrunpath=/var/lib/${module_name}
automxwsgipath=/usr/lib/${module_name}
logdir=/var/log
ssldir=/usr/local/ssl
certdir=${ssldir}/certs
vardir=/var/certs

apache2file=/etc/config.d/apache2
apache2modwsgifile=/etc/config.d/apache2-mod_wsgi
apache2_mods_enabled_file=/etc/apache2/mods-enabled/${module_name}.conf
basefile=/etc/config.d/base
mailfile=/etc/config.d/mail
nextcloudfile=/etc/config.d/nextcloud
owncloudfile=/etc/config.d/owncloud
config_file=/etc/config.d/${module_name}
check_script=/var/install/bin/automx-check-account
package_file=/var/install/packages/${module_name}
generate_config_file=/etc/${module_name}.conf
tmpfile=/var/run/${module_name}-tmp.$$
automxrunfile=${automxrunpath}/${module_name}
logfile=${logdir}/${module_name}.log
logrotatefile=/etc/logrotate.d/${module_name}
apache_user='wwwrun'
apache_group='nogroup'
apache_modpath=`get_apache2_module_path`
apache_wsgipath=`get_apache2_wsgi_path`
apache_mod_headers=${apache_modpath}/mod_headers.so
apache_mod_rewrite=${apache_modpath}/mod_rewrite.so
apache_mod_wsgi=${apache_wsgipath}/mod_wsgi.so

# read config file
. ${config_file}

if [ -f ${package_file} ]
then
    package_version="v`grep "<version>" ${package_file} | sed 's#<version>\(.*\)</version>#\1#'`"
else
    package_version="version could not be read"
fi

if [ -z "${AUTOMX_ORDER}" ]
then
    AUTOMX_ORDER='imap-pop3-smtp'
fi

if [ -z "${AUTOMX_CALDAV_CARDDAV_SERVER}" ]
then
    if [ -f ${nextcloudfile} ]
    then
        AUTOMX_CALDAV_CARDDAV_SERVER='nextcloud'
    elif [ -f ${owncloudfile} ]
    then
        AUTOMX_CALDAV_CARDDAV_SERVER='owncloud'
    else
        AUTOMX_CALDAV_CARDDAV_SERVER=''
    fi
fi

# check installed OpenSSL version
# OpenSSL 1.0.2l-fips  25 May 2017
openssl_version=`openssl version | sed 's/^openssl \([0-9.]*\).*$/\1/i'`

proxy_connect=''
if [ "`compare_version --text "${openssl_version}" '1.1'`" = 'old' ]
then
    # old OpenSSL version: < v1.1.0, e.g. v1.0.2
    openssl_separator=' '

    if [ -n "${HTTP_PROXY}" ]
    then
        myecho --pack ${module_name} --error "proxy usage is not supported by this OpenSSL version!"
        myanykey --pack ${module_name}
        exit 1
    fi
else
    # new OpenSSL version: >= v1.1.0
    openssl_separator='='

    if [ -n "${HTTP_PROXY}" ]
    then
        proxy_connect="-proxy ${HTTP_PROXY}"

        if [ -n "${HTTP_PROXY_USER}" ]
        then
            # a proxy authentication has been requested.
            myecho --pack ${module_name} --error "proxy authentication is not supported by the OpenSSL command!"
            myanykey --pack ${module_name}
            exit 1
        fi
    fi
fi

if [ -n "${AUTOMX_APACHE2_PREFIX}" ]
then
    # prefix configuration file name to influence the
    # load order of the virtual host configuration
    apache2_mods_enabled_file=/etc/apache2/mods-enabled/${AUTOMX_APACHE2_PREFIX}-${module_name}.conf
fi

silent=''
if called_from_install --pack "${module_name}"
then
    silent='--quiet'
fi

case $1 in
    *-check-dns )
        if [ "${START_AUTOMX}" = 'yes' ]
        then
            create_run_config
            . ${automxrunfile}

            check_dns_resolution
            myecho --pack ${module_name} ""
            check_dns_srv_entries
        fi
        ;;
    * )
        myecho --pack ${module_name} "version: ${package_version}"

        if [ "${START_AUTOMX}" = 'yes' ]
        then
            create_run_config
            . ${automxrunfile}

            WEB_INSTALLED='none'
            WSGI_INSTALLED='none'
            MAIL_INSTALLED='none'
            CERT_INSTALLED='none'
            CLOUD_INSTALLED='none'

            if check_installed_apache2
            then
                # apache2
                WEB_INSTALLED='apache2'
            fi

            if check_installed_apache2_wsgi
            then
                # apache2
                WSGI_INSTALLED='apache2wsgi'
            fi

            if check_installed_mail
            then
                # mail
                MAIL_INSTALLED='mail'
            fi

            if check_installed_cloud
            then
                # mail
                CLOUD_INSTALLED="${AUTOMX_CALDAV_CARDDAV_SERVER}"
            fi

            myecho --pack ${module_name} -n "certificate file: "

            if [ -z "${AUTOMX_CERT_NAME}" ]
            then
                # certificate
                myecho --pack ${module_name} --warn "<not set, check configuration>"
            elif [ ! -f "${certdir}/${AUTOMX_CERT_NAME}.pem" ]
            then
                myecho --pack ${module_name} --warn "file '${certdir}/${AUTOMX_CERT_NAME}.pem' doesn't exist!"
            else
                CERT_INSTALLED='certs'
                myecho --pack ${module_name} "${AUTOMX_CERT_NAME}"
            fi

            if [ "${WEB_INSTALLED}" = 'apache2' -a "${WSGI_INSTALLED}" = 'apache2wsgi' -a "${MAIL_INSTALLED}" = 'mail' -a "${CERT_INSTALLED}" = 'certs' ]
            then
                check_server_certificate

                idx=1
                access_control_ip=''
                while [ ${idx} -le ${IP_ETH_N} ]
                do
                    eval network='$IP_ETH_'${idx}'_NETWORK'
                    eval netmask='$IP_ETH_'${idx}'_NETMASK'

                    if [ -z "${access_control_ip}" ]
                    then
                        access_control_ip="${network}/`netcalc netmaskbits ${netmask}`"
                    else
                        access_control_ip="${access_control_ip} ${network}/`netcalc netmaskbits ${netmask}`"
                    fi

                    idx=`expr ${idx} + 1`
                done

                myecho --pack ${module_name} -n "access allowed from: "

                if [ -n "${AUTOMX_ACCESS_CONTROL_IP}" ]
                then
                    myecho --pack ${module_name} "${AUTOMX_ACCESS_CONTROL_IP}"
                else
                    myecho --pack ${module_name} "${access_control_ip}"
                fi

                if check_dns_resolution
                then
                    check_dns_srv_entries

                    if [ -z "${AUTOMX_DOCUMENT_ROOT}" ]
                    then
                        AUTOMX_DOCUMENT_ROOT="`echo "${APACHE2_DOCUMENT_ROOT}" | sed 's#/ *$##'`/${module_name}"
                    fi

                    # create web directories
                    for DNAME in ${AUTOMX_DOCUMENT_ROOT}/htdocs ${AUTOMX_DOCUMENT_ROOT}/log
                    do
                        mkdir -p ${DNAME}
                    done

                    # create symbolic link ${AUTOMX_DOCUMENT_ROOT}/automx -> /usr/lib/automx
                    if [ -d ${automxwsgipath} ]
                    then
                        # remove an existing file, link, directory, etc.
                        if [ -f ${AUTOMX_DOCUMENT_ROOT}/${module_name} -o -d ${AUTOMX_DOCUMENT_ROOT}/${module_name} ]
                        then
                            rm -f ${AUTOMX_DOCUMENT_ROOT}/${module_name}
                        fi

                        ln -sf ${automxwsgipath}/ ${AUTOMX_DOCUMENT_ROOT}/${module_name}
                    fi

                    # create configuration
                    create_conf
                    create_apache2_wsgi_conf
                    create_logrotate
                    set_access_rights

                    restart_apache2_service
                fi
            fi
        else
            myecho --pack ${module_name} --warn "package has been deactivated, removing configuration file(s) ..."
            create_logrotate

            if [ -f ${apache2_mods_enabled_file} ]
            then
                rm -f ${apache2_mods_enabled_file}
                restart_apache2_service
            fi
        fi
        ;;
esac

#========================================================================================
# end
#========================================================================================
exit 0
